(function($) {

	$.gamesRegistry('register', 'presentation', new (function() {

		this.options = {
			view_width : Infinity,
			view_height : Infinity,
			view_background_img : '',
			slides : [],
		};

		this.actualSlide = 1;
		this.total_slides = 0;

		this.clickEnable = true;

		this.resetGame = function() {
			this.actualSlide = 1;
		};

		this.showEl = function(el) {
			self.clickEnable = false;
			var show_el = $(el).attr('show_el');
			if (show_el !== undefined) {
				$('.is_hidden').hide();
				$('#' + show_el, $(el).parent()).fadeIn('slow', function() {
					if (show_el === 'TEXT_OK')
						$('.next_slide').show();
				});
				return true;
			}
			return false;
		};

		this.nextSlide = function() {
			$('.next_slide').hide();
			var self = this;
			self.clickEnable = false;
			if (this.actualSlide < this.total_slides) {
				$('.refresh.button').fadeOut('slow');
				$('.sound.button').fadeOut('slow');

				$('#slide-' + self.actualSlide).fadeOut('slow', function() {
					self.actualSlide++;
					if (self.actualSlide >= 6) {
						// $('#repeat_game').fadeIn();
						$('.refresh.button').fadeIn('slow');
					} else {
						$('.refresh.button').fadeIn('slow');
						$('.sound.button').fadeIn('slow');
					}
					$('#slide-' + self.actualSlide).fadeIn('slow', function() {
						self.clickEnable = true;
						$('.is_hidden').hide();
						
						$('[tabindex]:visible').sort(function(a, b) {
							var contentA = parseInt($(a).attr('tabindex'));
							var contentB = parseInt($(b).attr('tabindex'));
							return (contentA < contentB) ? -1 : (contentA > contentB) ? 1 : 0;
						}).first().focus();
					});
				});
			}
		};

		this.prevSlide = function() {
			// if (this.clickEnable === false)
			// return;
			var self = this;
			self.clickEnable = false;
			if (this.actualSlide > 0) {
				$('.refresh.button').fadeOut('slow');
				$('.sound.button').fadeOut('slow');
				$('#slide-' + self.actualSlide).fadeOut('slow', function() {
					self.actualSlide--;
					if (self.actualSlide > 1) {
						$('.refresh.button').fadeIn('slow');
						$('.sound.button').fadeIn('slow');
					}
					$('#slide-' + self.actualSlide).fadeIn('slow', function() {
						self.clickEnable = true;
						$('.presentation-img').removeClass('presentation-img-ok');
						$('.presentation-img').removeClass('presentation-img-bad');
					});
				});
			}
		};

		this.gotoSlide = function(no) {
			var self = this;
			self.clickEnable = false;

			$('#slide-' + self.actualSlide).fadeOut('slow', function() {
				self.actualSlide = no;
				$('#slide-' + self.actualSlide).fadeIn('slow', function() {
					self.clickEnable = true;
					$('.presentation-img').removeClass('presentation-img-ok');
					$('.presentation-img').removeClass('presentation-img-bad');
				});
			});
		};

		this.addBack = function(slide, no) {
			if (no === 1 || no === this.total_slides)
				return;
			var self = this;

			var back = $('<a href="javascript: void(0)" title="Previous" alt="Previous" class="left button" tabindex="6"></a>');

			back.click(function() {
				self.prevSlide();
			});

			// var obj = {
			// x : 30,
			// y : 750,
			// width : 150,
			// height : 150
			// };
			//
			// var area = new absoluteArea(obj);
			// area.applyAt(back, {
			// view_width : this.options.view_width,
			// view_height : this.options.view_height
			// });

			slide.append(back);
		};

		this.addNext = function() {
			var self = this;

			var next = $('<a href="javascript: void(0)" title="Next" alt="Next" class="right button next_slide" tabindex="7" style="display: none;"></a>');

			next.click(function() {
				self.nextSlide();
			});

			// var obj = {
			// x : 1420,
			// y : 745,
			// width : 150,
			// height : 150
			// };
			//
			// var area = new absoluteArea(obj);
			// area.applyAt(next, {
			// view_width : this.options.view_width,
			// view_height : this.options.view_height
			// });

			$('#playground').append(next);

			next = $('<a href="javascript: void(0)" title="start" alt="Start" id="start-page-start-button" tabindex="1"></a>');
			$('#slide-1').append(next);

			$('#start-page-start-button').click(function() {
				self.nextSlide();
			});
		};

		this.addRepeat = function(slide, no) {
			if (no !== this.total_slides)
				return;
			var obj = {
				x : 0,
				y : 600,
				width : this.options.view_width
			};

			var area = new absoluteArea(obj);
		};

		this.playSound = function(ok) {
			var sound;

			if (ok) {
				var soundId = Math.floor(Math.random() * this.options.ok_sounds.length);
				sound = this.options.ok_sounds[soundId];
			} else {
				var soundId = Math.floor(Math.random() * this.options.fail_sounds.length);
				console.log(soundId);
				sound = this.options.fail_sounds[soundId];
			}

			if ($('.sound.button.off').length === 1) {
				return;
			}

			if (this.snd_instance !== undefined) {
				this.snd_instance.stop();
			}

			if (snd_instance !== undefined) {
				snd_instance.pause();
			}

			if (this.characterSound) {
				this.characterSound.stop();
			}

			this.characterSound = createjs.Sound.play(sound, createjs.Sound.INTERUPT_LATE);
			this.characterSound.addEventListener("complete", function() {
				if (snd_instance !== undefined) {
					snd_instance.resume();
				}
			});
		};

		this.init = function(data, endCallback) {
			var self = this;
			$.extend(this.options, data);

			var background = preload.getResult(self.options.view_background_img).src;
			$('#playground').css('background-image', 'url("' + background + '")');

			self.total_slides = Object.keys(self.options.slides).length;

			var i = 1;
			var slide_hasClickableElem = false;
			$.each(self.options.slides, function(index, slide) {

				slide_hasClickableElem = false;
				var slide_item = $('<div class="presentation-elem presentation-slide" id="slide-' + i + '"></div>');

				// images
				var j = 1;
				$.each(slide.images, function(index, image) {
					var image_item = $('<div class="presentation-elem presentation-img" id="image-' + i + '-' + j + '"></div>');
					var img = preload.getResult(image.image);

					if (this.width === undefined)
						this.width = img.width;
					if (this.height === undefined)
						this.height = img.height;

					if (this.show !== undefined && this.show !== '') {
						image_item.attr('show_el', this.show);
					}

					image_item.css('background-image', 'url("' + img.src + '")');

					if (image.class_name !== undefined)
						image_item.addClass(image.class_name);
					if (image.scale !== undefined) {
						this.width += (this.width * image.scale / 100);
						this.height += (this.height * image.scale / 100);
					}

					var area = new absoluteArea(this);
					area.applyAt(image_item, {
						view_width : self.options.view_width,
						view_height : self.options.view_height
					});

					if (image.clickable) {
						image_item.css('cursor', 'pointer');
						slide_hasClickableElem = true;
						image_item.click(function() {
							if (self.clickEnable === false)
								return;
							if (image.ok === true) {
								self.clickEnable = false;
								if (self.options.sound_ok !== undefined && self.options.sound_ok !== '')
									createjs.Sound.play(self.options.sound_ok, createjs.Sound.INTERUPT_LATE);
								image_item.removeClass('animated bounce').addClass('animated bounce');

								if (self.showEl(image_item)) {
									window.setTimeout(function() {
										image_item.removeClass('animated bounce');
									}, 1300);
									return;
								}

								window.setTimeout(function() {
									image_item.removeClass('animated bounce');

									setTimeout(function() {
										self.nextSlide();
									}, 1000);
								}, 1300);

							} else {
								self.clickEnable = false;
								// zastopowac dzwieki
								if (self.options.sound_bad !== undefined && self.options.sound_bad !== '')
									createjs.Sound.play(self.options.sound_bad, createjs.Sound.INTERUPT_LATE);
								image_item.removeClass('animated shake').addClass('animated shake');
								self.showEl(image_item);
								window.setTimeout(function() {
									image_item.removeClass('animated shake');
									self.clickEnable = true;
								}, 1300);
							}
						});
					}

					slide_item.append(image_item);

					j++;
				});

				// texts
				j = 1, tabindex = 1;
				$.each(slide.texts, function(index, text) {
					var text_item = null;

					if (text.clickable) {
						text_item = $('<a href="javascript: void(0)" />');
						text_item.attr('tabindex', tabindex++);
					} else {
						text_item = $('<div />');
					}

					text_item.addClass("presentation-elem presentation-txt").attr('id', 'txt-' + i + '-' + j).html(text.text || '').attr("alt", text.text);

					if (text.img) {
						var imgsrc = preload.getResult(text.img).src;
						var img = $('<img class="slite-text-img">').attr('src', imgsrc);
						img.attr('alt', text.alt || '')
						
						if (text.imgClass) {
							img.addClass(text.imgClass);
						}
						text_item.append(img);
					}
					if (text.class !== undefined)
						text_item.addClass(text.class);
					if (text.color !== undefined)
						text_item.css('color', text.color);
					if (text.font !== undefined)
						text_item.css('font-family', text.font);
					if (text.size !== undefined)
						text_item.css('font-size', text.size);
					if (text.align !== undefined)
						text_item.css('text-align', text.align);
					if (text.hidden !== undefined && text.hidden === true)
						text_item.css('display', 'none');
					if (text.id !== undefined && text.id !== '') {
						text_item.attr('id', text.id);
						text_item.addClass('is_hidden');
					}

					if (this.show !== undefined && this.show !== '') {
						text_item.attr('show_el', this.show);
					}

					var area = new absoluteArea(this);
					area.applyAt(text_item, {
						view_width : self.options.view_width,
						view_height : self.options.view_height
					});

					if (text.clickable) {
						text_item.css('cursor', 'pointer');
						text_item.addClass('clicable-txt');
						slide_hasClickableElem = true;
						text_item.click(function() {
							if (self.clickEnable === false)
								return;
							if (text.ok === true) {

								self.clickEnable = false;
								self.playSound(true);
								text_item.removeClass('animated bounce').addClass('animated bounce');

								if (self.showEl(text_item)) {
									window.setTimeout(function() {
										text_item.removeClass('animated bounce');
									}, 1300);
									return;
								}

								window.setTimeout(function() {
									text_item.removeClass('animated bounce');

									setTimeout(function() {
										self.nextSlide();
									}, 1000);
								}, 1300);

							} else {
								self.clickEnable = false;
								// zastopowac dzwieki
								self.playSound(false);
								text_item.removeClass('animated shake').addClass('animated shake');
								self.showEl(text_item);
								window.setTimeout(function() {
									text_item.removeClass('animated shake');
									self.clickEnable = true;
								}, 1300);

							}
						});
					}

					slide_item.append(text_item);
					j++;
				});

				self.addBack(slide_item, i);
				self.addRepeat(slide_item, i);

				$('#playground').append(slide_item);

				i++;

				if (slide_hasClickableElem === false) {

					// slide_item.click(function(){
					// if (self.clickEnable==false) return;
					// self.nextSlide(slide_item);
					// })
				}
			});

			this.addNext();

			$('#slide-1').fadeIn('slow', function() {

			});

			return this;
		};
	})());

})(jQuery);